@@ -2,11 +2,9 @@
 
 use anyhow::{bail, Result};
 use proc_macro2::Span;
-use syn::parse::Error;
-use syn::spanned::Spanned;
 use syn::{
-    Attribute, DeriveInput, Field, Fields, Ident, Lit, Meta, MetaList, MetaNameValue, NestedMeta,
-    Type,
+    parse::Error, spanned::Spanned, Attribute, DeriveInput, Field, Fields, Ident, Lit, Meta,
+    MetaList, MetaNameValue, NestedMeta, Type,
 };
 
 pub struct Template {
@@ -17,7 +15,30 @@ pub struct Template {
 pub enum TemplateSource {
     File(String),
     Resource(String),
-    String(String),
+    Xml(String),
+    #[cfg(feature = "blueprint")]
+    Blueprint(String),
+}
+
+impl TemplateSource {
+    fn from_string_source(source: String) -> Result<Self> {
+        for c in source.chars() {
+            #[cfg(feature = "blueprint")]
+            if c.is_ascii_alphabetic() {
+                // blueprint code starts with some alphabetic letters
+                return Ok(Self::Blueprint(source));
+            } else if c == '<' {
+                // xml tags starts with '<' symbol
+                return Ok(Self::Xml(source));
+            }
+            #[cfg(not(feature = "blueprint"))]
+            if c == '<' {
+                // xml tags starts with '<' symbol
+                return Ok(Self::Xml(source));
+            }
+        }
+        bail!("Unknown language")
+    }
 }
 
 pub fn parse_template_source(input: &DeriveInput) -> Result<Template> {
@@ -70,7 +91,7 @@ pub fn parse_template_source(input: &DeriveInput) -> Result<Template> {
     let source = match ident.as_ref() {
         "file" => TemplateSource::File(v),
         "resource" => TemplateSource::Resource(v),
-        "string" => TemplateSource::String(v),
+        "string" => TemplateSource::from_string_source(v)?,
         s => bail!("Unknown enum meta {}", s),
     };
     Ok(Template {
